import { CacheExpiration } from './CacheExpiration.js';
import { ExpirationPlugin, ExpirationPluginOptions } from './ExpirationPlugin.js';
import './_version.js';
/**
 * @module workbox-expiration
 */
export { CacheExpiration, ExpirationPlugin, ExpirationPluginOptions };
